# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import os
import shutil
import stat

import photosort.test as test
from photosort import media

TEST_DIR_FMT = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
TEST_FILE_FMT = "%(year)04d%(month)02d%(day)02d%(hour)02d" \
                "%(minute)02d%(second)02d_"


class TestPhotoMedia(test.TestCase):

    def setUp(self):
        super(TestPhotoMedia, self).setUp()
        self.img1 = self.get_data_path('media1/img1.jpg')
        self.img1dup = self.get_data_path('media1/img1_dup.jpg')
        self.photo = media.MediaFile.build_for(self.img1)

    def test_hash_creation(self):
        expected_hash = "a35de42abad366d0f6232a4abd0404c8 " \
            "- 2013-08-24 13:05:52"
        self.assertEqual(self.photo.hash(), expected_hash)

        # check for hasher non being re-started
        same_photo = media.MediaFile.build_for(self.img1)
        self.assertEqual(same_photo.hash(), expected_hash)

    def test_datetime(self):
        expected_datetime = "2013-08-24 13:05:52"  # it must come from EXIF

        self.assertEqual(str(self.photo.datetime()), expected_datetime)

    def test_equal_checking(self):
        self.assertTrue(self.photo.is_equal_to(self.img1dup))

    def test_datatime_dir(self):
        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        dir_str = self.photo.calculate_datetime(dir_fmt)
        self.assertEqual(dir_str, "2013/2013_08_24")

    def test_get_filename(self):
        self.assertEqual(self.photo.get_filename(), 'img1.jpg')

    def test_rename(self):
        tmpfile = self._temp_dir + '/' + self.photo.get_filename()
        tmpfile_renamed = self._temp_dir + '/R' + self.photo.get_filename()
        tmpfile_mode = 0o666

        shutil.copy(self.photo.get_path(), tmpfile)

        photo_t = media.MediaFile.build_for(tmpfile)

        photo_t.rename_as(tmpfile_renamed, tmpfile_mode)
        self.assertTrue(self.photo.is_equal_to(tmpfile_renamed))

        file_mode = os.stat(tmpfile_renamed)[stat.ST_MODE] & 0o777
        self.assertEqual(file_mode, tmpfile_mode)

    def test_move_to_directory(self):
        tmpfile = self._temp_dir + '/' + self.photo.get_filename()

        shutil.copy(self.photo.get_path(), tmpfile)
        photo_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(self._temp_dir + '/2013', ignore_errors=True)
        photo_t.move_to_directory_with_date(self._temp_dir, TEST_DIR_FMT)

        self.assertTrue(self.photo.is_equal_to(
            self._temp_dir + '/2013/2013_08_24/img1.jpg'))

        shutil.copy(self.photo.get_path(), tmpfile)
        photo_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(self._temp_dir + '/2013', ignore_errors=True)
        photo_t.move_to_directory_with_date(self._temp_dir, TEST_DIR_FMT,
                                            TEST_FILE_FMT)

        self.assertTrue(self.photo.is_equal_to(
            self._temp_dir + '/2013/2013_08_24/20130824130552_img1.jpg'))

    def test_move_to_existing_directory_pattern(self):
        tmpfile = self._temp_dir + '/' + self.photo.get_filename()

        shutil.copy(self.photo.get_path(), tmpfile)
        photo_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(self._temp_dir + '/2013', ignore_errors=True)

        default_dir = photo_t.locate_output_directory(self._temp_dir,
                                                      TEST_DIR_FMT)
        default_dir_tagged = default_dir + "_tagged_with_something"
        os.makedirs(default_dir_tagged)
        # after renaming an existing directory, it should figure out and
        # use that one destination
        self.assertEqual(default_dir_tagged,
                         photo_t.locate_output_directory(self._temp_dir,
                                                         TEST_DIR_FMT))

        photo_t.move_to_directory_with_date(self._temp_dir, TEST_DIR_FMT)

        self.assertTrue(self.photo.is_equal_to(
            self._temp_dir + '/2013/2013_08_24_tagged_with_something/img1.jpg')
        )

    def test_move_to_existing_directory_pattern_file_fmt(self):

        tmpfile = self._temp_dir + '/' + self.photo.get_filename()
        shutil.copy(self.photo.get_path(), tmpfile)
        photo_t = media.MediaFile.build_for(tmpfile)

        default_dir = photo_t.locate_output_directory(self._temp_dir,
                                                      TEST_DIR_FMT)
        default_dir_tagged = default_dir + "_tagged_with_something"
        os.makedirs(default_dir_tagged)
        photo_t.move_to_directory_with_date(self._temp_dir, TEST_DIR_FMT,
                                            TEST_FILE_FMT)

        file_path = os.path.join(self._temp_dir,
                                 '2013',
                                 '2013_08_24_tagged_with_something',
                                 '20130824130552_img1.jpg')
        self.assertTrue(self.photo.is_equal_to(file_path))


if __name__ == '__main__':
    test.test.main()



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import os.path
import tempfile
import unittest


class TestCase(unittest.TestCase):
    @staticmethod
    def get_data_path(file_path):
        return os.path.join(os.path.dirname(__file__), 'data', file_path)

    def setUp(self):
        _temp_dir = tempfile.TemporaryDirectory(prefix=self.id())
        self._temp_dir = _temp_dir.name
        self.addCleanup(_temp_dir.cleanup)



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import mock
import photosort.test as test
from photosort import walk


class TestWalkForMedia(test.TestCase):

    def setUp(self):
        self.media1 = self.get_data_path('media1')

    def test_directory_inspection(self):

        # We need to patch the timecheck (not modified in last 30 seconds)
        # in CI (recent checkout)
        with mock.patch("photosort.walk.WalkForMedia._file_is_ready") as m:
            instance = m.return_value
            instance.method.return_value = True
            walker = walk.WalkForMedia(self.media1)
            files = [file for root, file in walker.find_media()]
            self.assertIn('img1.jpg', files)

    def test_directory_inspection_file_not_ready(self):

        with mock.patch("photosort.walk.WalkForMedia._file_is_ready") as m:
            instance = m.return_value
            instance.method.return_value = False
            walker = walk.WalkForMedia(self.media1)
            files = [file for root, file in walker.find_media()]
            self.assertNotIn('img1', files)

    def test_ignores(self):
        pass


if __name__ == '__main__':
    test.test.main()



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import os
import shutil
import stat
import time

import photosort.test as test
from photosort import media


class TestMovieMedia(test.TestCase):

    def setUp(self):
        super(TestMovieMedia, self).setUp()
        self.mov1 = self.get_data_path('media2/mov1.mp4')
        self.mov1dup = self.get_data_path('media2/mov1_dup.mp4')
        self.mov1_mtime = os.path.getmtime(self.mov1)
        self.movie = media.MediaFile.build_for(self.mov1)

    def test_hash_creation(self):
        expected_hash = "d41d8cd98f00b204e9800998ecf8427e"
        self.assertEqual(self.movie.hash(), expected_hash)

        # check for hasher non being re-started
        same_movie = media.MediaFile.build_for(self.mov1)
        self.assertEqual(same_movie.hash(), expected_hash)

    def test_datetime(self):
        mtime = time.localtime(self.mov1_mtime)
        expected_datetime = time.strftime(
            "%Y-%m-%d %H:%M:%S", mtime)  # it must come from exif data

        self.assertIn(expected_datetime, str(self.movie.datetime()))

    def test_equal_checking(self):
        self.assertTrue(self.movie.is_equal_to(self.mov1dup))

    def test_datatime_dir(self):
        mtime = time.localtime(self.mov1_mtime)
        year = "%d" % mtime[0]
        month = "%02d" % mtime[1]
        day = "%02d" % mtime[2]

        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        dir_str = self.movie.calculate_datetime(dir_fmt)
        expected_dir = os.path.join(year, year + '_' + month + '_' + day)
        self.assertEqual(dir_str, expected_dir)

    def test_get_filename(self):
        self.assertEqual(self.movie.get_filename(), 'mov1.mp4')

    def test_rename(self):

        tmpfile = self._temp_dir + '/' + self.movie.get_filename()
        tmpfile_renamed = self._temp_dir + '/R' + self.movie.get_filename()
        tmpfile_mode = 0o666

        shutil.copy(self.movie.get_path(), tmpfile)

        movie_t = media.MediaFile.build_for(tmpfile)

        movie_t.rename_as(tmpfile_renamed, tmpfile_mode)
        self.assertTrue(self.movie.is_equal_to(tmpfile_renamed))

        file_mode = os.stat(tmpfile_renamed)[stat.ST_MODE] & 0o777
        self.assertEqual(file_mode, tmpfile_mode)

    def test_move_to_directory(self):
        tmpfile = self._temp_dir + '/' + self.movie.get_filename()
        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        file_fmt = "%(year)04d%(month)02d%(day)02d%(hour)02" \
            "d%(minute)02d%(second)02d_"

        shutil.copy(self.movie.get_path(), tmpfile)
        mov_mtime = os.path.getmtime(tmpfile)
        mtime = time.localtime(mov_mtime)
        year = "%d" % mtime[0]
        month = "%02d" % mtime[1]
        day = "%02d" % mtime[2]
        hour = "%02d" % mtime[3]
        minute = "%02d" % mtime[4]
        second = "%02d" % mtime[5]
        movie_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(self._temp_dir + '/' + year, ignore_errors=True)
        movie_t.move_to_directory_with_date(self._temp_dir, dir_fmt)

        expected_filename = os.path.join(
            self._temp_dir, year, year + '_' + month + '_' + day, 'mov1.mp4')
        self.assertTrue(self.movie.is_equal_to(expected_filename))

        shutil.copy(self.movie.get_path(), tmpfile)
        mov_mtime = os.path.getmtime(tmpfile)
        mtime = time.localtime(mov_mtime)
        year = "%d" % mtime[0]
        month = "%02d" % mtime[1]
        day = "%02d" % mtime[2]
        hour = "%02d" % mtime[3]
        minute = "%02d" % mtime[4]
        second = "%02d" % mtime[5]
        movie_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(self._temp_dir + '/' + year, ignore_errors=True)
        movie_t.move_to_directory_with_date(self._temp_dir, dir_fmt, file_fmt)

        expected_filename = os.path.join(
            self._temp_dir, year, year + '_' + month + '_' + day,
            year + month + day + hour + minute + second + '_mov1.mp4')
        self.assertTrue(self.movie.is_equal_to(expected_filename))


if __name__ == '__main__':
    test.test.main()



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import datetime
import filecmp
import hashlib
import logging
import os
import os.path
import shutil
import stat
import sys


class MediaFile:

    def __init__(self, filename):
        self._filename = filename
        self._file_type = MediaFile.guess_file_type(filename)
        self._hash = None

    @staticmethod
    def guess_file_type(filename):

        extension = filename.lower().split('.')[-1]
        if extension in (
                'heic', 'jpeg', 'jpg', 'cr2', 'raw', 'png', 'arw', 'thm', 'orf'):
            return 'photo'
        if extension in ('m4v','mpeg', 'mpg', 'mov', 'mp4', 'avi'):
            return 'movie'
        return 'unknown'

    @staticmethod
    def build_for(filename):

        file_type = MediaFile.guess_file_type(filename)
        if file_type == 'photo':
            from photosort import \
                photo  # delayed import to avoid circular dependencies
            return photo.Photo(filename)
        else:
            return MediaFile(filename)

    def get_filename(self):
        return os.path.basename(self._filename)

    def get_directory(self):
        return os.path.dirname(self._filename)

    def get_path(self):
        return self._filename

    def hash(self, hasher=None, blocksize=65536):
        if self._hash is not None:
            return self._hash

        if hasher is None:
            hasher = hashlib.md5()

        with open(self._filename, 'rb') as afile:
            buf = afile.read(blocksize)
            while len(buf) > 0:
                hasher.update(buf)
                buf = afile.read(blocksize)

            self._hash = hasher.hexdigest()
            return self._hash

    def datetime(self):

        ct1 = os.path.getmtime(self._filename)
        ct2 = os.path.getctime(self._filename)

        ct = min(ct1, ct2)  # it can differ from windows to UN*X

        return datetime.datetime.fromtimestamp(ct)

    def __str__(self):

        s = "[%s file hash=%s date=%s]" % (
            self._file_type, self.hash(), self.datetime())
        return s

    def is_equal_to(self, filename):

        try:
            result = filecmp.cmp(self._filename, filename, shallow=True)
            return result
        except OSError:
            logging.info(
                "Comparing to %s, file didn't exist anymore, "
                "erased or moved?", filename)
            return False

    def type(self):
        return self._file_type

    def makedirs_f(self, path, mode):
        paths = os.path.split(path)

        total_path = ''
        for directory in paths:
            total_path = os.path.join(total_path, directory)
            if os.path.isdir(total_path):
                continue

            os.mkdir(total_path, mode | stat.S_IXUSR)

    def rename_as(self, new_filename, file_mode=0o774):

        try:
            self.makedirs_f(os.path.dirname(new_filename), file_mode)
        except OSError as e:
            logging.error("Unable to move: %s (%s)", new_filename, e)
            return False

        try:
            shutil.move(self._filename, new_filename)
            os.chmod(new_filename, file_mode)
        except OSError as e:
            logging.error("Unable to move: %s", e)
            return False

        return True

    def calculate_datetime(self, format):
        dt = self.datetime()
        data = {'year': dt.year, 'month': dt.month, 'day': dt.day,
                'hour': dt.hour, 'minute': dt.minute, 'second': dt.second}

        return format % data

    def locate_output_directory(self, directory, dir_format):
        default = os.path.join(directory, self.calculate_datetime(dir_format))
        parts = os.path.split(default)
        last_dir = parts[-1]
        top_dir = os.path.join(*parts[:-1])
        # try to figure out if a directory with the same pattern and a suffix
        # added by the user exists already (tagged directory support)
        try:
            for item in os.listdir(top_dir):
                file_path = os.path.join(top_dir, item)
                if os.path.isdir(file_path) and item.startswith(last_dir):
                    return file_path
        except FileNotFoundError:
            return default
        return default

    def move_to_directory_with_date(self, directory, dir_format,
                                    file_format='', file_mode=0o774):

        out_dir = self.locate_output_directory(directory, dir_format)

        try:
            os.mkdir(out_dir)
            os.chmod(out_dir, file_mode | stat.S_IXUSR)
        except OSError:
            pass  # it already exists

        if file_format != "":
            file_prefix = self.calculate_datetime(
                file_format) + self.get_filename()
        else:
            file_prefix = self.get_filename()
        new_filename = out_dir + "/" + file_prefix
        logging.info("moving %s to %s", self._filename, new_filename)

        if self.rename_as(new_filename, file_mode):
            self._filename = new_filename
            return True
        else:
            return False


if __name__ == "__main__":
    file = MediaFile.build_for(sys.argv[1])
    print(file)



#!/usr/bin/env python3
import setuptools
from photosort import photosort

setuptools.setup(
    name='photosort',
    version=photosort.VERSION,
    description='Picture inbox simplified',
    author='Miguel Angel Ajo Pelayo',
    author_email='miguelangel@ajo.es',
    url='https://github.com/mangelajo/photosort',
    packages=setuptools.find_packages(exclude=['ez_setup']),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'photosort = photosort.photosort:main'
        ]},
    install_requires=['pyaml', 'Pillow'],

    data_files=[('etc', ['etc/photosort.yml'])],
    test_suite='photosort.test.testcases'
)
